import tkinter as tk
from tkinter.ttk import *
from tkinter import filedialog
from PIL import ImageTk, Image
import webbrowser
import pandas as pd
import numpy as np
import paphra_tktable as tktable
import sys
import os
import xlrd
from xlrd import open_workbook
import xlsxwriter
import math
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import QFile, QFileInfo, Qt, QAbstractTableModel
from PyQt5.QtGui import QStandardItem, QStandardItemModel
from PyQt5.QtWidgets import QApplication, QHeaderView, QTableView
from datetime import datetime

### How to make an .exe .. from console
# pyinstaller --onefile -w "full path filename"
# -- onefile to put the libraries in the exe
# -w to hide the console window when the program starts execution


class FreezeTableWidget(QTableView):
#############################
### class function to freeze columns and rows of a QTable found in:
### https://gist.github.com/gdementen/21a78ac56258c07dbc1072b806a5097a
### and
### https://github.com/baoboa/pyqt5/blob/master/examples/itemviews/frozencolumn/frozencolumn.py
############################
##    from PyQt5.QtCore import QFile, QFileInfo, Qt
##    from PyQt5.QtGui import QStandardItem, QStandardItemModel
##    from PyQt5.QtWidgets import QApplication, QHeaderView, QTableView


    def __init__(self, model):
        super(FreezeTableWidget, self).__init__()
        self.setModel(model)
        self.frozenTableView = QTableView(self)
        self.init()

        self.horizontalHeader().sectionResized.connect(self.updateSectionWidth)
        self.verticalHeader().sectionResized.connect(self.updateSectionHeight)
        self.frozenTableView.verticalScrollBar().valueChanged.connect(
            self.verticalScrollBar().setValue)
        self.verticalScrollBar().valueChanged.connect(
            self.frozenTableView.verticalScrollBar().setValue)
#        self.layout().setAlignment(Qt.AlignTop)

    def init(self):
        self.frozenTableView.setModel(self.model())
        self.frozenTableView.setFocusPolicy(Qt.NoFocus)
        self.frozenTableView.verticalHeader().hide()
#        self.frozenTableView.verticalHeader().setSectionResizeMode(
 #           QHeaderView.Interactive)

#        self.frozenTableView.verticalHeader().setSectionResizeMode(3) ### ADDED TO MAKE VERTICAL HEADER EXAND IN 2 LINES
#        self.frozenTableView.horizontalHeader().setSectionResizeMode(3)
#        self.frozenTableView.horizontalHeader().setSectionResizeMode(
#            QHeaderView.Interactive)
        self.viewport().stackUnder(self.frozenTableView)

        self.frozenTableView.setStyleSheet('''
            QTableView { border: none;
                         background-color: #8EDE21;
                         selection-background-color: #999;
            }''') # for demo purposes

        self.frozenTableView.setSelectionModel(self.selectionModel())
        for col in range(1, self.model().columnCount()):
            self.frozenTableView.setColumnHidden(col, True)
        self.frozenTableView.setColumnWidth(0, self.columnWidth(0))

        self.frozenTableView.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.frozenTableView.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        
        self.frozenTableView.show()
        self.updateFrozenTableGeometry()
        self.setHorizontalScrollMode(self.ScrollPerPixel)
        self.setVerticalScrollMode(self.ScrollPerPixel)
        self.frozenTableView.setVerticalScrollMode(self.ScrollPerPixel)

    def updateSectionWidth(self, logicalIndex, oldSize, newSize):
        if logicalIndex == 0:
            self.frozenTableView.setColumnWidth(0, newSize)
            self.updateFrozenTableGeometry()

    def updateSectionHeight(self, logicalIndex, oldSize, newSize):
        self.frozenTableView.setRowHeight(logicalIndex, newSize)

    def resizeEvent(self, event):
        super(FreezeTableWidget, self).resizeEvent(event)
        self.updateFrozenTableGeometry()

    def moveCursor(self, cursorAction, modifiers):
        current = super(FreezeTableWidget, self).moveCursor(cursorAction, modifiers)
        if (cursorAction == self.MoveLeft and
                self.current.column() > 0 and
                self.visualRect(current).topLeft().x() <
                    self.frozenTableView.columnWidth(0)):
            newValue = (self.horizontalScrollBar().value() +
                        self.visualRect(current).topLeft().x() -
                        self.frozenTableView.columnWidth(0))
            self.horizontalScrollBar().setValue(newValue)
        return current

    def scrollTo(self, index, hint):
        if index.column() > 0:
            super(FreezeTableWidget, self).scrollTo(index, hint)

    def updateFrozenTableGeometry(self):
        self.frozenTableView.setGeometry(
                self.verticalHeader().width() + self.frameWidth(),
                self.frameWidth(), self.columnWidth(0),
                self.viewport().height() + self.horizontalHeader().height())

class pandasModel(QAbstractTableModel):
##################
### This class function converts pandas dataframes to QAbstractTableModel
### It was found in:
### "https://gist.githubusercontent.com/DataSolveProblems/972884bb9a53d5b2598e8674acc9e8ab/
### raw/055f0a9e1f0cb4ed56552799dc5926ffed99bbec/Display%20Pandas%20DataFrame%20-%20PyQt5"
### after this discussion subject:
### https://stackoverflow.com/questions/44603119/how-to-display-a-pandas-data-frame-with-pyqt5-pyside2
#################    
    def __init__(self, data):
        QAbstractTableModel.__init__(self)
        self._data = data

    def rowCount(self, parent=None):
        return self._data.shape[0]

    def columnCount(self, parnet=None):
        return self._data.shape[1]

    def data(self, index, role=Qt.DisplayRole):
        if index.isValid():
            if role == Qt.DisplayRole:
                return str(self._data.iloc[index.row(), index.column()])
            if role == Qt.TextAlignmentRole: ### added 2 lines to get center aligned cells
                return Qt.AlignCenter
        return None

    def headerData(self, col, orientation, role):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            return self._data.columns[col]
        return None

def callback1(event):
    webbrowser.open_new(r"http://fishlab.bio.auth.gr/")
def callback2(event):
    webbrowser.open_new(r"http://river.bio.auth.gr/")

def  replace_window(root):
    """Destroy current window, create new window"""
    global current_window
    if current_window is not None:
        current_window.destroy()
    current_window = tk.Toplevel(root, height=900, width = 700)
    current_window.geometry('900x700')
    

    # if the user kills the window via the window manager,
    # exit the application. 
    current_window.wm_protocol("WM_DELETE_WINDOW", root.destroy)

    return current_window

def bSwitch(buttonName, state):
    bNames=["Input", "Datashow", "Metrics", "Export","Clear","Quit","File"]
    bIds=[4,5,6,7,8,9,10]
    bId=bIds[bNames.index(buttonName)]
    canvas.itemconfigure(bId, state=state)


def input_choose():
##    Επιλέγει αρχείο δειγμάτων και φορτώνει τα δεδομένα  σε μορφή wb, df και data
    global rFileName, wb, dfin, data, file_label
    rFileName = filedialog.askopenfilename(filetypes = (("Microsoft Excel 97-2003 Worksheet", "*.xls"),("Microsoft Excel Worksheets", "*.xlsx")))
    try:  
        wb=open_workbook(rFileName) #   χρειάζεται γιατί το Metrics ξαναδιαβάζει το Excel
#                                       και δεν χρησιμοποιει το dataframe
        dfin = pd.read_excel(rFileName, header=None)
        data=dfin.values
        filename = os.path.basename(rFileName)
        file_label = tk.Label(root, text = "Imported data file: "+filename, anchor = 'w',
                              width = 50, activebackground = "#33B5E5", font=('Arial', '14', 'bold'))
        file_label_window = canvas.create_window(470, 350, anchor='c', window=file_label)
        bSwitch("Input",'hidden')
        bSwitch("Datashow",'normal')
        bSwitch("Metrics",'normal')
        bSwitch("Export",'hidden')
        bSwitch("Clear",'normal')
    except:
        bSwitch("Input",'normal')
        bSwitch("Datashow",'hidden')
        bSwitch("Metrics",'hidden')
        bSwitch("Clear",'hidden')
        pass

#    #print(open_workbook(filename))
   
def sampleShow():
    df=dfin
    filename = os.path.basename(rFileName)
    for i in range(1,len(df.columns)):
       for j in range(0,len(df[i])):
            a=df[i][j]
#            print(i, j, a, type(a))  
            if j==2 and type(a)==float: #convert excel date to dd/mm/yyyy format
                a=xlrd.xldate_as_datetime(a, 0).date().strftime("%d/%m/%Y")
                df[i][j]=a
            if j==2 and type(a)==datetime:
                a=a.strftime("%d/%m/%Y")
                df[i][j]=a
            if j==4 :
                df[i][j]=str(df[i][j-3])+"\n"+str(df[i][j-2])+"\n"+str(df[i][j-1])+"\n"+str(df[i][j])
    df[0][4]="Sample\nDate\nRiver type\nHabitat"
    df, df.columns = df[5:] , df.iloc[4]
    frozenshow(filename,df)

def interpret(grades, totest):
    num = 5
    for limit in grades:
        if totest > limit:
            break
        else:
            num = num-1
    res=num
    return(res);


def makefamilies():
# ---------------
#----------------------------- Import Scores from 
# ----------------
## the filecontains Abundancy indices for selected families
#'Οικογένεια', '0-1p', '1,001-10%', '>10%', '', ''
# MAKE FAMILIES DICTIONARY
#
#    epilogh = input("Θέλετε να διαβάσετε παραμέτρους από αρχείο: NAI/OXI")
    epilogh="OXI"
    if epilogh=="ΝΑΙ":    
        try:
            wb = open_workbook('Scores_ASPT_HESY_BMWP_IBMWP.xls')
        except FileNotFoundError:
            wb = open_workbook('Scores_ASPT_HESY_BMWP_IBMWP.xlsx')
        except:
            pass
        ## ------------  Read workbook lignes 
        scores = []
        for sheet in wb.sheets():
            number_of_rows = sheet.nrows
            number_of_columns = sheet.ncols
            rows = []
            for row in range(1, number_of_rows):
                values = []
                for col in range(number_of_columns):
                    value  = (sheet.cell(row,col).value)
                    try:
                        value = str(int(value))
                    except ValueError:
                        pass
                    finally:
                        values.append(value)
        #        item = Arm(*values)
                score = values
                scores.append(score)
        #### MAKE FAMILIES DICTIONARY
        FAMILIES={}
        for score in scores:
            FAMILIES[score[4].strip()]=score[1:13]

    else:
        FAMILIES={'Acroloxidae': ['Gastropoda', 'Pulmonata', '', 'Acroloxidae', '50', '53', '56', '1', '10', 'M', '0', '0'], \
                  'Aeshnidae': ['Insecta', 'Odonata', '', 'Aeshnidae', '80', '86', '90', '1', '10', 'S', '8', '8'], \
                  'Ameletidae': ['Insecta', 'Ephemeroptera', '', 'Ameletidae', '0', '0', '0', '1', '10', '', '0', '0'], \
                  'Ancylus': ['Gastropoda', 'Pulmonata', '', 'Ancylus', '50', '53', '56', '1', '10', 'M', '6', '6'], \
                  'Anthomyiidae': ['Insecta', 'Diptera', '', 'Anthomyiidae', '40', '38', '35', '1', '10', 'T', '4', '0'],\
                  'Aphelocheiridae': ['Insecta', 'Hemiptera', '', 'Aphelocheiridae', '100', '110', '120', '1', '10', 'S', '10', '10'], \
                  'Asellidae': ['Malacostraca', 'Isopoda', '', 'Asellidae', '30', '25', '20', '1', '10', 'T', '3', '3'], \
                  'Astacidae': ['Malacostraca', 'Decapoda', '', 'Astacidae', '80', '86', '90', '1', '10', 'S', '8', '8'], \
                  'Athericidae': ['Insecta', 'Diptera', '', 'Athericidae', '80', '86', '90', '1', '10', 'S', '10', '0'], \
                  'Atyidae': ['Malacostraca', 'Decapoda', '', 'Atyidae', '50', '53', '56', '1', '10', 'M', '6', '0'], \
                  'Baetidae': ['Insecta', 'Ephemeroptera', '', 'Baetidae', '40', '38', '35', '1', '10', 'T', '4', '4'], \
                  'Belostomatidae': ['Insecta', 'Hemiptera', '', 'Belostomatidae', '30', '25', '20', '1', '10', 'T', '0', '0'], \
                  'Beraeidae': ['Insecta', 'Trichoptera', '', 'Beraeidae', '100', '110', '120', '1', '10', 'S', '10', '10'], \
                  'Bithyniidae': ['Gastropoda', 'Sorbeoconcha', '', 'Bithyniidae ', '30', '25', '20', '1', '10', 'T', '3', '0'], \
                  'Blephariceridae': ['Insecta', 'Diptera', '', 'Blephariceridae', '100', '110', '120', '1', '10', 'S', '10', '0'], \
                  'Brachycentridae': ['Insecta', 'Trichoptera', '', 'Brachycentridae', '100', '110', '120', '1', '10', 'S', '10', '10'], \
                  'Caenidae': ['Insecta', 'Ephemeroptera', '', 'Caenidae', '50', '53', '56', '1', '10', 'M', '4', '7'], \
                  'Calamoceratidae': ['Insecta', 'Trichoptera', '', 'Calamoceratidae', '100', '110', '120', '1', '10', 'S', '10', '0'], \
                  'Calopterygidae': ['Insecta', 'Odonata', '', 'Calopterygidae', '70', '75', '78', '1', '10', 'M', '8', '8'], \
                  'Capniidae': ['Insecta', 'Plecoptera', '', 'Capniidae', '100', '110', '120', '1', '10', 'S', '10', '10'],\
                  'Ceratopogonidae': ['Insecta', 'Diptera', '', 'Ceratopogonidae', '60', '64', '67', '1', '10', 'M', '4', '0'], \
                  'Chaoboridae': ['Insecta', 'Diptera', '', 'Chaoboridae', '20', '12', '3', '1', '10', 'T', '0', '0'], \
                  'Chironomidae': ['Insecta', 'Diptera', '', 'Chironomidae', '30', '25', '20', '10', '20', 'T', '2', '2'],\
                  'Chloroperlidae': ['Insecta', 'Plecoptera', '', 'Chloroperlidae', '100', '110', '120', '1', '10', 'S', '10', '10'],\
                  'Chrysomelidae': ['Insecta', 'Coleoptera', '', 'Chrysomelidae', '40', '38', '35', '1', '10', 'T', '4', '5'],\
                  'Coenagrionidae': ['Insecta', 'Odonata', '', 'Coenagrionidae', '30', '25', '20', '1', '10', 'T', '6', '6'], \
                  'Cordulegastridae': ['Insecta', 'Odonata', '', 'Cordulegastridae', '70', '75', '78', '1', '10', 'M', '8', '8'],\
                  'Corduliidae': ['Insecta', 'Odonata', '', 'Corduliidae', '80', '86', '90', '1', '10', 'S', '8', '8'], \
                  'Corixidae': ['Insecta', 'Hemiptera', '', 'Corixidae', '30', '25', '20', '1', '10', 'T', '3', '5'],\
                  'Corophiidae': ['Malacostraca', 'Amphipoda', '', 'Corophiidae', '50', '53', '56', '1', '10', 'M', '6', '6'], \
                  'Culicidae': ['Insecta', 'Diptera', '', 'Culicidae', '20', '12', '3', '1', '10', 'T', '2', '0'],\
                  'Curculionidae': ['Insecta', 'Coleoptera', '', 'Curculionidae', '40', '38', '35', '1', '10', 'T', '4', '5'],\
                  'Cylindrotomidae': ['Insecta', 'Diptera', '', 'Cylindrotomidae', '0', '0', '0', '1', '10', '', '0', '0'],\
                  'Dendrocoelidae': ['Platyhelminthes', 'Tricladida', '', 'Dendrocoelidae', '50', '53', '56', '1', '10', 'M', '5', '5'],\
                  'Dixidae': ['Insecta', 'Diptera', '', 'Dixidae', '80', '86', '90', '1', '10', 'S', '4', '0'],\
                  'Dolichopodidae': ['Insecta', 'Diptera', '', 'Dolichopodidae', '40', '38', '35', '1', '10', 'T', '4', '0'], \
                  'Dreissenidae': ['Bivalvia', 'Veneroida', '', 'Dreissenidae', '0', '0', '0', '1', '10', '', '0', '0'], \
                  'Dryopidae': ['Insecta', 'Coleoptera', '', 'Dryopidae', '50', '53', '56', '1', '10', 'M', '5', '5'], \
                  'Dugesiidae': ['Platyhelminthes', 'Tricladida', '', 'Dugesiidae', '50', '53', '56', '1', '10', 'M', '5', '0'], \
                  'Dytiscidae': ['Insecta', 'Coleoptera', '', 'Dytiscidae', '30', '25', '20', '1', '10', 'T', '3', '5'],\
                  'Ecnomidae': ['Insecta', 'Trichoptera', '', 'Ecnomidae', '80', '86', '100', '1', '10', 'S', '7', '0'], \
                  'Elmidae': ['Insecta', 'Coleoptera', 'Elminthidae', 'Elmidae', '60', '64', '67', '1', '10', 'M', '5', '5'], \
                  'Elminthidae': ['Insecta', 'Coleoptera', '', 'Elminthidae', '60', '64', '67', '1', '10', 'M', '5', '5'], \
                  'Empididae': ['Insecta', 'Diptera', '', 'Empididae', '60', '64', '67', '1', '10', 'M', '4', '0'], \
                  'Ephemerellidae': ['Insecta', 'Ephemeroptera', '', 'Ephemerellidae', '40', '38', '35', '1', '10', 'T', '7', '10'],\
                  'Ephemeridae': ['Insecta', 'Ephemeroptera', '', 'Ephemeridae', '80', '86', '90', '1', '10', 'S', '10', '10'], \
                  'Ephydridae': ['Insecta', 'Diptera', '', 'Ephydridae', '20', '12', '3', '1', '10', 'T', '2', '0'], \
                  'Erpobdellidae': ['Hirudinea', 'Arhynchobdellida', '', 'Erpobdellidae', '20', '12', '3', '1', '10', 'T', '3', '3'], \
                  'Euphaeidae': ['Insecta', 'Odonata', '', 'Euphaeidae', '0', '0', '0', '1', '10', '', '0', '0'], \
                  'Ferrissia': ['Gastropoda', '', '', 'Ferrissia', '0', '0', '0', '1', '10', '', '6', '0'], \
                  'Gammaridae': ['Malacostraca', 'Amphipoda', '', 'Gammaridae', '50', '53', '56', '1', '10', 'M', '6', '6'], \
                  'Gerridae': ['Insecta', 'Hemiptera', '', 'Gerridae', '30', '25', '20', '1', '10', 'T', '3', '5'], \
                  'Glossiphoniidae': ['Hirudinea', 'Rhynchobdellida', '', 'Glossiphoniidae', '40', '38', '35', '1', '10', 'T', '3', '3'], \
                  'Glossosomatidae': ['Insecta', 'Trichoptera', '', 'Glossosomatidae', '80', '86', '90', '1', '10', 'S', '8', '0'], \
                  'Goeridae': ['Insecta', 'Trichoptera', '', 'Goeridae', '90', '97', '100', '1', '10', 'S', '10', '10'], \
                  'Gomphidae': ['Insecta', 'Odonata', '', 'Gomphidae', '60', '64', '67', '1', '10', 'M', '8', '8'],\
                  'Gyrinidae': ['Insecta', 'Coleoptera', '', 'Gyrinidae', '80', '86', '90', '1', '10', 'S', '3', '5'], \
                  'Haliplidae': ['Insecta', 'Coleoptera', '', 'Haliplidae', '40', '38', '35', '1', '10', 'T', '4', '5'], \
                  'Hebridae': ['Insecta', 'Hemiptera', '', 'Hebridae', '30', '25', '20', '1', '10', 'T', '0', '0'],\
                  'Helicopsychidae': ['Insecta', 'Trichoptera', '', 'Helicopsychidae', '100', '110', '120', '1', '10', 'S', '0', '0'], \
                  'Helophoridae': ['Insecta', 'Coleoptera', '', 'Helophoridae', '50', '53', '56', '1', '10', 'M', '5', '0'],\
                  'Heptageniidae': ['Insecta', 'Ephemeroptera', '', 'Heptageniidae', '80', '86', '90', '1', '10', 'S', '10', '10'], \
                  'Hirudinidae': ['Hirudinea', 'Arhynchobdellida', '', 'Hirudinidae', '30', '25', '20', '1', '10', 'T', '3', '3'],\
                  'Hydrachnidae': ['Arachnida', 'Trombidiformes', '', 'Hydrachnidae', '40', '38', '35', '1', '10', 'T', '4', '0'],\
                  'Hydraenidae': ['Insecta', 'Coleoptera', '', 'Hydraenidae', '80', '86', '90', '1', '10', 'S', '5', '0'], \
                  'Hydrobiidae': ['Gastropoda', 'Sorbeoconcha', '', 'Hydrobiidae', '70', '75', '78', '1', '10', 'M', '3', '3'], \
                  'Hydrochidae': ['Insecta', 'Coleoptera', '', 'Hydrochidae', '50', '53', '56', '1', '10', 'M', '5', '0'],\
                  'Hydrometridae': ['Insecta', 'Hemiptera', '', 'Hydrometridae', '30', '25', '20', '1', '10', 'T', '3', '5'], \
                  'Hydrophilidae': ['Insecta', 'Coleoptera', 'Spercheidae', 'Hydrophilidae', '30', '25', '20', '1', '10', 'T', '3', '5'], \
                  'Spercheidae': ['Insecta', 'Coleoptera', '', 'Spercheidae', '30', '25', '20', '1', '10', 'T', '3', '5'], \
                  'Hydropsychidae': ['Insecta', 'Trichoptera', '', 'Hydropsychidae', '50', '53', '56', '1', '10', 'M', '5', '5'],\
                  'Hydroptilidae': ['Insecta', 'Trichoptera', '', 'Hydroptilidae', '40', '38', '35', '1', '10', 'T', '6', '6'],\
                  'Hydroscaphidae': ['Insecta', 'Coleoptera', '', 'Hydroscaphidae', '40', '38', '35', '1', '10', 'T', '0', '0'],\
                  'Hygrobiidae': ['Insecta', 'Coleoptera', '', 'Hygrobiidae', '30', '25', '20', '1', '10', 'T', '3', '5'], \
                  'Isonychiidae': ['Insecta', 'Ephemeroptera', '', 'Isonychiidae', '50', '53', '56', '1', '10', 'M', '0', '0'],\
                  'Lepidostomatidae': ['Insecta', 'Trichoptera', '', 'Lepidostomatidae', '100', '110', '120', '1', '10', 'S', '10', '10'],\
                  'Leptoceridae': ['Insecta', 'Trichoptera', '', 'Leptoceridae', '80', '86', '90', '1', '10', 'S', '10', '10'], \
                  'Leptophlebiidae': ['Insecta', 'Ephemeroptera', '', 'Leptophlebiidae', '80', '86', '90', '1', '10', 'S', '10', '10'],\
                  'Lestidae': ['Insecta', 'Odonata', '', 'Lestidae', '80', '86', '90', '1', '10', 'S', '8', '8'], \
                  'Leuctridae': ['Insecta', 'Plecoptera', '', 'Leuctridae', '90', '97', '100', '1', '10', 'S', '10', '10'], \
                  'Libellulidae': ['Insecta', 'Odonata', '', 'Libellulidae', '80', '86', '90', '1', '10', 'S', '8', '8'],\
                  'Limnephilidae': ['Insecta', 'Trichoptera', '', 'Limnephilidae', '80', '86', '90', '1', '10', 'S', '7', '7'], \
                  'Limoniidae': ['Insecta', 'Diptera', '', 'Limoniidae', '40', '38', '35', '1', '10', 'T', '4', '0'],\
                  'Lymnaeidae': ['Gastropoda', 'Pulmonata', '', 'Lymnaeidae', '20', '12', '3', '1', '10', 'T', '3', '3'],\
                  'Mesoveliidae': ['Insecta', 'Hemiptera', '', 'Mesoveliidae', '30', '25', '20', '1', '10', 'T', '3', '5'],\
                  'Molannidae': ['Insecta', 'Trichoptera', '', 'Molannidae', '100', '110', '120', '1', '10', 'S', '10', '10'],\
                  'Muscidae': ['Insecta', 'Diptera', '', 'Muscidae', '20', '12', '3', '1', '10', 'T', '0', '0'], \
                  'Mysidae': ['Malacostraca', 'Decapoda', '', 'Mysidae', '0', '0', '0', '1', '10', '', '0', '0'],\
                  'Naucoridae': ['Insecta', 'Hemiptera', '', 'Naucoridae', '30', '25', '20', '1', '10', 'T', '3', '5'], \
                  'Nemouridae': ['Insecta', 'Plecoptera', '', 'Nemouridae', '80', '86', '90', '1', '10', 'S', '7', '7'],\
                  'Nepidae': ['Insecta', 'Hemiptera', '', 'Nepidae', '30', '25', '20', '1', '10', 'T', '3', '5'], \
                  'Neritidae': ['Gastropoda', 'Neritopsina', '', 'Neritidae', '60', '64', '67', '1', '10', 'M', '6', '6'],\
                  'Nevrorthidae': ['Insecta', 'Neuroptera', 'Neurorthidae', 'Nevrorthidae', '0', '0', '0', '1', '10', '', '0', '0'],\
                  'Neurorthidae': ['Insecta', 'Neuroptera','' , 'Neurorthidae', '0', '0', '0', '1', '10', '', '0', '0'],\
                  'Noteridae': ['Insecta', 'Coleoptera', '', 'Noteridae', '0', '0', '0', '1', '10', '', '3', '0'], \
                  'Notonectidae': ['Insecta', 'Hemiptera', '', 'Notonectidae', '30', '25', '20', '1', '10', 'T', '3', '5'],\
                  'Odontoceridae': ['Insecta', 'Trichoptera', '', 'Odontoceridae', '100', '110', '120', '1', '10', 'S', '10', '10'],\
                  'Oligochaeta': ['Oligochaeta', '', '', 'Oligochaeta', '30', '25', '20', '10', '20', 'T', '1', '1'],\
                  'Oligoneuriidae': ['Insecta', 'Ephemeroptera', '', 'Oligoneuriidae', '50', '53', '56', '1', '10', 'M', '5', '0'],\
                  'Osmylidae': ['Insecta', 'Neuroptera', '', 'Osmylidae', '0', '0', '0', '1', '10', '', '0', '0'],\
                  'Ostracoda': ['', 'Ostracoda', '', 'Ostracoda', '30', '38', '35', '1', '10', 'T', '3', '0'],\
                  'Palaemonidae': ['Malacostraca', 'Decapoda', '', 'Palaemonidae', '0', '0', '0', '1', '10', '', '6', '0'],\
                  'Perlidae': ['Insecta', 'Plecoptera', '', 'Perlidae', '90', '97', '100', '1', '10', 'S', '10', '10'], \
                  'Perlodidae': ['Insecta', 'Plecoptera', '', 'Perlodidae', '90', '97', '100', '1', '10', 'S', '10', '10'],\
                  'Philopotamidae': ['Insecta', 'Trichoptera', '', 'Philopotamidae', '80', '86', '90', '1', '10', 'S', '8', '8'], \
                  'Phryganeidae': ['Insecta', 'Trichoptera', '', 'Phryganeidae', '100', '110', '120', '1', '10', 'S', '10', '10'], \
                  'Physidae': ['Gastropoda', 'Pulmonata', '', 'Physidae', '30', '25', '20', '1', '10', 'T', '3', '3'], \
                  'Piscicolidae': ['Hirudinea', 'Rhynchobdellida', '', 'Piscicolidae', '40', '38', '35', '1', '10', 'T', '4', '4'], \
                  'Planariidae': ['Platyhelminthes', 'Tricladida', '', 'Planariidae', '50', '53', '56', '1', '10', 'M', '5', '5'],\
                  'Planorbidae': ['Gastropoda', 'Pulmonata', '', 'Planorbidae', '20', '12', '3', '1', '10', 'T', '3', '3'], \
                  'Platycnemididae': ['Insecta', 'Odonata', '', 'Platycnemididae', '60', '64', '67', '1', '10', 'M', '6', '6'], \
                  'Pleidae': ['Insecta', 'Hemiptera', '', 'Pleidae', '30', '25', '20', '1', '10', 'T', '3', '5'], \
                  'Polycentropodidae': ['Insecta', 'Trichoptera', '', 'Polycentropodidae', '80', '86', '90', '1', '10', 'S', '7', '7'], \
                  'Polymitarcyidae': ['Insecta', 'Ephemeroptera', '', 'Polymitarcyidae', '50', '53', '56', '1', '10', 'M', '5', '0'], \
                  'Potamanthidae': ['Insecta', 'Ephemeroptera', '', 'Potamanthidae', '70', '75', '78', '1', '10', 'M', '10', '10'], \
                  'Potamonidae': ['Malacostraca', 'Decapoda', 'Potamidae', 'Potamonidae', '80', '86', '90', '1', '10', 'S', '0', '0'],\
                  'Potamidae': ['Malacostraca', 'Decapoda','' , 'Potamidae', '80', '86', '90', '1', '10', 'S', '0', '0'],\
                  'Prosopistomatidae': ['Insecta', 'Ephemeroptera', '', 'Prosopistomatidae', '0', '0', '0', '1', '10', '', '7', '0'],\
                  'Psephenidae': ['Insecta', 'Coleoptera', '', 'Psephenidae', '0', '0', '0', '1', '10', '', '3', '0'],\
                  'Psychodidae': ['Insecta', 'Diptera', '', 'Psychodidae', '50', '53', '56', '1', '10', 'M', '4', '0'], \
                  'Psychomyiidae': ['Insecta', 'Trichoptera', 'Psychomyidae', 'Psychomyiidae', '80', '86', '90', '1', '10', 'S', '8', '8'],\
                  'Psychomyidae': ['Insecta', 'Trichoptera','' , 'Psychomyidae', '80', '86', '90', '1', '10', 'S', '8', '8'],\
                  'Ptilocolepidae': ['Insecta', 'Trichoptera', '', 'Ptilocolepidae ', '40', '38', '35', '1', '10', 'T', '0', '0'],\
                  'Ptychopteridae': ['Insecta', 'Diptera', '', 'Ptychopteridae', '0', '0', '0', '1', '10', '', '4', '0'],\
                  'Rhagionidae': ['Insecta', 'Diptera', '', 'Rhagionidae', '20', '12', '3', '1', '10', 'T', '4', '0'], \
                  'Rhyacophilidae': ['Insecta', 'Trichoptera', '', 'Rhyacophilidae', '80', '86', '90', '1', '10', 'S', '7', '7'],\
                  'Scathophagidae': ['Insecta', 'Diptera', '', 'Scathophagidae', '0', '0', '0', '1', '10', '', '4', '0'],\
                  'Sciomyzidae': ['Insecta', 'Diptera', '', 'Sciomyzidae', '0', '0', '0', '1', '10', '', '4', '0'], \
                  'Scirtidae': ['Insecta', 'Coleoptera', 'Helodidae', 'Scirtidae', '80', '86', '90', '1', '10', 'S', '3', '5'],\
                  'Helodidae': ['Insecta', 'Coleoptera', '', 'Helodidae', '80', '86', '90', '1', '10', 'S', '3', '5'],\
                  'Sericostomatidae': ['Insecta', 'Trichoptera', '', 'Sericostomatidae', '90', '97', '100', '1', '10', 'S', '10', '10'],\
                  'Sialidae': ['Insecta', 'Megaloptera', '', 'Sialidae', '80', '86', '90', '1', '10', 'S', '4', '4'], \
                  'Simuliidae': ['Insecta', 'Diptera', '', 'Simuliidae', '50', '53', '56', '1', '10', 'M', '5', '5'], \
                  'Siphlonuridae': ['Insecta', 'Ephemeroptera', '', 'Siphlonuridae', '100', '110', '120', '1', '10', 'S', '10', '10'], \
                  'Sphaeriidae': ['Bivalvia', 'Veneroida', '', 'Sphaeriidae', '30', '25', '20', '1', '10', 'T', '3', '3'], \
                  'Sphaeromatidae': ['Malacostraca', 'Isopoda', '', 'Sphaeromatidae', '0', '0', '0', '1', '10', '', '0', '0'], \
                  'Stratiomyidae': ['Insecta', 'Diptera', '', 'Stratiomyidae', '70', '75', '78', '1', '10', 'M', '4', '0'],\
                  'Syrphidae': ['Insecta', 'Diptera', '', 'Syrphidae', '10', '2', '1', '1', '10', 'T', '1', '0'],\
                  'Tabanidae': ['Insecta', 'Diptera', '', 'Tabanidae', '60', '64', '67', '1', '10', 'M', '4', '0'],\
                  'Taeniopterygidae': ['Insecta', 'Plecoptera', '', 'Taeniopterygidae', '80', '86', '90', '1', '10', 'S', '10', '10'], \
                  'Thaumaleidae': ['Insecta', 'Diptera', '', 'Thaumaleidae', '20', '12', '3', '1', '10', 'T', '2', '0'], \
                  'Thiaridae': ['Gastropoda', 'Neotaenioglossa', '', 'Thiaridae', '30', '25', '20', '1', '10', 'T', '6', '0'],\
                  'Tipulidae': ['Insecta', 'Diptera', '', 'Tipulidae', '40', '38', '35', '1', '10', 'T', '5', '5'], \
                  'Tubificidae': ['Oligochaeta', '', '', 'Tubificidae', '10', '2', '1', '1', '10', 'T', '1', '1'],\
                  'Uenoidae': ['Insecta', 'Trichoptera', '', 'Uenoidae', '100', '110', '120', '1', '10', 'S', '10', '0'], \
                  'Unionidae': ['Bivalvia', 'Unionida', '', 'Unionidae', '60', '64', '67', '1', '10', 'M', '6', '6'], \
                  'Valvatidae': ['Gastropoda', 'Heterostropha', '', 'Valvatidae', '10', '2', '1', '1', '10', 'T', '3', '3'], \
                  'Veliidae': ['Insecta', 'Hemiptera', 'Microveliidae', 'Veliidae', '30', '25', '20', '1', '10', 'T', '3', '0'],\
                  'Microveliidae': ['Insecta', 'Hemiptera', '', 'Microveliidae', '30', '25', '20', '1', '10', 'T', '3', '0'],\
                  'Viviparidae': ['Gastropoda', 'Architaenioglossa  ', '', 'Viviparidae', '60', '64', '67', '1', '10', 'M', '6', '6'], \
                  'Unknown': ['', '', '', 'Unknown', '0', '0', '0', '1', '10', '', '0', '0']}
    return FAMILIES
#------- ----------------------------------------- END makefamilies() -----------------


def  Metrics():
    global result
##############               Οικογένειες που θα αθροίζονται για τα ομαδοποιημένα αποτελέσματα
#### Για τα EPT και EPTC
# όσες ανήκουν στις παρακάτω Κλάσεις
    pclasses=["Bivalvia","Gastropoda","Hirudinea","Platyhelminthes","Oligochaeta","Arachnida"]
# και όσες ανήκουν στις παρακάτω Τάξεις
    porders=["Coleoptera", "Diptera","Ephemeroptera", "Hemiptera", "Megaloptera","Neuroptera","Odonata","Plecoptera","Trichoptera","Amphipoda","Isopoda","Decapoda","Ostracoda"]    
### 
    Sel_EPTDs=['Athericidae','Brachycentridae','Dixidae','Dolichopodidae','Empididae','Ephemeridae','Goeridae','Heptageniidae','Leptophlebiidae','Limnephilidae','Nemouridae','Odontoceridae','Polycentropodidae','Stratiomyidae']

    FAMILIES=makefamilies()
    

    ## ------------  Read workbook lignes
    scores = []
    for sheet in wb.sheets():
        #print(sheet.name)
        number_of_rows = sheet.nrows
        number_of_columns = sheet.ncols
        if number_of_rows !=149:
            if number_of_rows>149 :msg="περισσότερες από 149 γραμμές"
            else:msg="λιγότερες από 149 γραμμές"
            #print("Ελέγξτε το αρχείο!! Έχει "+msg)


        rows = []
        datafamilies=[] # λίστα οικογενειών στο αρχείο ανάγνωσης δειγμάτων
        result=[] # λίστα λιστών αποτελεσμάτων με κεφαλίδες για εξαγωγή
            
        for col in range(number_of_columns):
    ## -------------------------------------  LOOP για κάθε στήλη Δηλαδη για κάθε σταθμό  ------------
            resultlist=[] # λίστα αποτελεσμάτων ανά σταθμό
           
            resindex=[] # Περιέχει την πρώτη γραμμή κεφαλίδων του αρχείου αποτελεσμάτων
            dataclasses=[] # κλάσεις των ταξινομικών ομάδων των δεδομένων
            dataorders=[]  # τάξεις
            values = [] # αριθμός ατόμων της ταξινομικής ομάδας
            taxa=0
            GROUPED={}
            SENSIBLES={}
    #        
            aa=sheet.cell(0,col).value
            Sample=sheet.cell(1,col).value
            Date=sheet.cell(2,col).value
            Rivertype=sheet.cell(3,col).value
            Habitat=sheet.cell(4,col).value

            for row in range(5, number_of_rows):
                value  = (sheet.cell(row,col).value)
                try:
                    value = int(value)
                except ValueError:
                    pass
                finally:
                    if col==0:
                        datafamilies.append(value)
#                        print(row)
#                        print(datafamilies[row-5], len(datafamilies))
                    if col>0 and int(value) > 0 and datafamilies[row-5] not in ["Tubificidae","Ancylus","Ferrissia"]:
                        taxa=taxa+1
                    values.append(value)

    #                             0      1           2        3           4
            resindex=resindex+['A/A','Sample code','Date','River type','Habitat'] #OK
    #                                       5                 6
            resindex=resindex+['Total Nb individuals','Total Nb taxa']
    #                                    7                          8                        9
            resindex=resindex+['Nb of families Sensitive', 'Nb of families Tolerant','Nb of families Medium'] 
    #                             10               11         12
            resindex=resindex+['% Sensitive','% Tolerant','% Medium']
    #                           13     14       15         16           17          18      19         
            resindex=resindex+['HES','AHES','HESgrade', 'AHESgrade', 'SemiHES', 'HESY2','Quality HESY2']
    #                           20          21          22          23          24                  25                      26
            resindex=resindex+['ASPT-2','logEPTDs+1','Taxa EPT','1-GOLD','Shannon-Wiener index', 'STAR ICMi','Quality STAR ICMi']
    #                           27          28          29        
            resindex=resindex+['IBMWP', 'IASPT', 'Quality IBMWP']
    #                           30      31          32  
            resindex=resindex+['BMWP','ASPT','Quality BMWP']
    #                               33                  34              35          36              37          38          39          40        
            resindex=resindex+['% Ephemeroptera','% Trichoptera','% Coleoptera','% Plecoptera','% Diptera','% Odonata','%Gastropoda','%Oligochaeta']
    #                                     41                          42                        43                              44                       45                     
            resindex=resindex+['Nb families Ephemeroptera','Nb of families Trichoptera','Nb of families Coleoptera','Nb of families Plecoptera', 'Nb of families Diptera']
    #                                    46                         47
            resindex=resindex+['Nb of families Odonata','Nb of families EPTC']
    #                           48          49
            resindex=resindex+['% EPT', '% EPTC']


            if col==0: result.append(resindex)

            if col>0:
                pis=[]
                #print(col, aa,Sample,Date,Rivertype,Habitat)
                #print(values)
                resultlist=resultlist+[aa,Sample,Date,Rivertype,Habitat] # 0:A/A',1:Sample code',2:Date',3:River type',4:Habitat
                
                sum_atoma=sum(list(values))
                resultlist=resultlist+[sum_atoma,taxa] #5:'Total Nb individuals',6:'Total Nb taxa'
                
    # Relative abundances P[i]
                pis=[values[i]/sum_atoma for i in range(len(values)) if values[i]>0]
    # ln(P[i])
                lnpis=[math.log(pis[i]) for i in range(len(pis)) if pis[i]!=0]
    # Shannon Diversity Index = SUM(P[i] * ln(P[i]))
                shannon =-sum(pis[i]*lnpis[i] for i in range(len(pis)))

                
    ### ################### ΥΠΟΛΟΓΙΣΜΟΣ ΤΩΝ SCORES HES & ASPT ###############
    # Βαθμολογίες ΕΣΑ/ΗΕS
    # Το άθροισμα των βαθμολογιών των οικογενειών σε κάθε δείγμα, ανάλογα με τη σχετική τους αφθονία,
    # αποτελεί την τιμή του Ελληνικού Σκορ Αξιολόγησης (ΕΣΑ) (ΗΕS) για το δείγμα αυτό.
    # Ωστόσο, ο ΕΣΑ επηρεάζεται από τον αριθμό των ταξινομικών ομάδων που συλλέχθηκαν στο δείγμα,
    # όπως και ο Βρετανικός BMWP.
    # Κρίθηκε, λοιπόν, αναγκαίος και ο υπολογισμός του μέσου όρου του ΕΣΑ (δηλ. ο ΜΕΣΑ) (ΑΗΕS),
    # ο οποίος είναι αντίστοιχος του Βρετανικού και Ισπανικού ΑSΡΤ
    # Μέσος Ορος Ανά Ταξινομική Ομάδα):
    # Στο ΕΣΑ οι Βαθμολογίες των ταξινομικών ομάδων μεταβάλλονται ανάλογα με τη σχετική τους αφθονία.
    # Οι βαθμολογίες των 13X3 κάθε δείγματος αθροίζονται δίνοντας την τιμή του ΕΣΑ.
                percent=[]
                scoring=[]
                IBMWPscoring=[] # τα σκόρ στον Ισπανικό δείκτη IBMWP
                BMWPscoring=[] # τα σκόρ στο δείκτη Λίλας
                taxaHESY=0 # αριθμός ταξινομικών ομάδων με Θετικό γινόμενο άτομα x σκόρ στο HESY
                taxaBMWP=0 # αριθμός ταξινομικών ομάδων με Θετικό γινόμενο άτομα x σκόρ στο BMWP
                Sensitivities=[]
    ##### ------------------------------  Για κάθε οικογένεια του δείγματος ---------------------            
                for fam in datafamilies:
                    classes=FAMILIES[fam][0] # η κλάση της οικογένειας
                    orders=FAMILIES[fam][1] # η τάξη της οικογένειας
                    synonyms=FAMILIES[fam][2] # το συνώνυμό της αν έχει
                    families=FAMILIES[fam][3] # η οικογένεια
                    HESYlows=FAMILIES[fam][4] #
                    HESYmids=FAMILIES[fam][5] #
                    HESYmaxs=FAMILIES[fam][6] #
                    HESYmid2s=FAMILIES[fam][7] #
                    HESYmax2s=FAMILIES[fam][8] #
                    Sensitivity=FAMILIES[fam][9] # ευαισθησία
                    IBMWPs=FAMILIES[fam][10] # σκόρ στο IBMWP
                    BMWPs=FAMILIES[fam][11] # σκορ στο BMWPs (LILA's)
                    Sensitivities.append(Sensitivity)
                    ind=datafamilies.index(fam) # θέση τηςοικογένειας στον πίνακα δεδομένων
                    percent.append(values[ind]/sum_atoma*100) # ποσοστό ατόμων οικογένειας στο σύνολο του δείγματος
    #                #print(percent[ind],HESYmax2s[ind])
                    if percent[ind] > float(HESYmax2s):
                        scoring.append(float(HESYmaxs))
    #                    #print("1st condition", percent[ind], HESYmax2s,HESYmaxs)
                    elif percent[ind] > float(HESYmid2s):
                        scoring.append(float(HESYmids))
    #                    #print("2nd condition", percent[ind], HESYmid2s,HESYmids)
                    elif percent[ind]!=0.0 :
                        scoring.append(float(HESYlows))
    #                    #print("3rd condition", percent[ind], HESYlows)
                    else :
                        scoring.append(0)

    # ----------------------- Υπολογισμός του BMWPs (Lila's) ----------------------
                    if values[ind] >0:
                        IBMWPscoring.append(int(IBMWPs))
                        BMWPscoring.append(int(BMWPs))
                    else:
                        IBMWPscoring.append(0)
                        BMWPscoring.append(0)
                    
                        
     #               #print("\t\t",fam,"\t",values[ind],"\t %8.2f"%(percent[ind]),"\t",scoring[ind])
     #               #print("% 2d"% (42))

                scoring_sum=sum(scoring[x] for x in range(0, len(datafamilies)))
                taxaHESY=len([x for x in scoring if x > 0])
                BMWPscoring_sum=sum(BMWPscoring[x] for x in range(0, len(datafamilies)))
                IBMWPscoring_sum=sum(IBMWPscoring[x] for x in range(0, len(datafamilies)))
                taxaIBMWP=len([x for x in IBMWPscoring if x > 0])
                taxaBMWP=len([x for x in BMWPscoring if x > 0])

### 15/21/2020 Update               
                try:
                    scoring_mean=scoring_sum/taxaHESY
                except ZeroDivisionError:
                    scoring_mean=0
                try:
                    BMWPscoring_mean=BMWPscoring_sum/taxaBMWP
                except ZeroDivisionError:
                    BMWPscoring_mean=0
                try:
                    IBMWPscoring_mean=IBMWPscoring_sum/taxaIBMWP
                except ZeroDivisionError:
                    IBMWPscoring_mean=0
###########
                    
                NbtaxaSensible=len([values[x] for x in range(0,len(values)) if Sensitivities[x]=='S' and values[x]>0])
                NbSensible=sum(values[x] for x in range(0,len(values)) if Sensitivities[x]=='S')
                pSensible=NbSensible/sum_atoma*100
                #print('Nb Taxa Sensible:', NbtaxaSensible, 'Nb Sensible:', NbSensible, '%Sensible: ', pSensible)
                NbtaxaMedium=len([values[x] for x in range(0,len(values)) if Sensitivities[x]=='M' and values[x]>0])
                NbMedium=sum(values[x] for x in range(0,len(values)) if Sensitivities[x]=='M')
                pMedium=NbMedium/sum_atoma*100
                #print('Nb Taxa Medium:', NbtaxaMedium,'Nb Medium:', NbMedium, '%Medium: ', pMedium)
                NbtaxaTolerant=len([values[x] for x in range(0,len(values)) if Sensitivities[x]=='T' and values[x]>0])
                NbTolerant=sum(values[x] for x in range(0,len(values)) if Sensitivities[x]=='T')
                pTolerant=NbTolerant/sum_atoma*100
                #print('Nb Taxa Tolerant:', NbtaxaTolerant, 'Nb Tolerant:', NbTolerant,'%Tolerant: ', pTolerant)
                
                resultlist=resultlist+[NbtaxaSensible,NbtaxaTolerant,NbtaxaMedium] # 7:NbSensible, 8:NbTolerant,9:NbMedium
                resultlist=resultlist+[pSensible,pTolerant,pMedium] # 10:%Sensible, 11:%Tolerant,12:%Medium
                
    ### Ελληνικό Σκορ Αξιολόγησης (ΕΣΑ) = HEllenic Scoring (HES)
    #   Μέσο Ελληνικό Σκορ Αξιολόγησης (ΜΕΣΑ) = Average Hellenic Scoring (AHES)
                HES=scoring_sum
                AHES=scoring_mean
                IBMWP=IBMWPscoring_sum
                IASPT=IBMWPscoring_mean
                BMWP=BMWPscoring_sum
                ASPT=BMWPscoring_mean
                ASPTminus2=ASPT-2

                
    #########################################################################            
                                
                #print("Άθροισμα ατόμων: ", sum_atoma)
                #print("Αριθμός οικογενειών:", taxa)
                #print("Αριθμός οικογενειών με σκόρ HESY:", taxaHESY)
                #print("Αριθμός οικογενειών με σκόρ BMWP:", taxaBMWP)
                #print("Αριθμός οικογενειών με σκόρ IBMWP:", taxaIBMWP)
                #print("Shannon Diversity index :", shannon)
                #print("Ελληνικό Σκορ Αξιολόγησης (ΕΣΑ) = HEllenic Scoring (HES):", HES)
                #print("Μέσο Ελληνικό Σκορ Αξιολόγησης (ΜΕΣΑ) = Average Hellenic Scoring (AHES): ", AHES)
                #print("British Monitoring Working Party (IBMWP) : ", IBMWP)
                #print("Average Score Per Taxon (IASPT): ", IASPT)
                #print("British Monitoring Working Party (BMWP) : ", BMWP)
                #print("Average Score Per Taxon (ASPT): ", ASPT)


    #### ΥΠΟΛΟΓΙΣΜΟΣ ΑΝΕΞΑΡΤΗΤΩΝ ΑΡΙΘΜΩΝ HES και AHES
    # Interpretation
    #(Habitat ="Μεγάλη"; 0 - 340    -   829     -   1325    -   1532    
    #                    |  1  |     2    |     3       |   4       |   5
                HESrich=[1532,1325,829,340]
                HESpoor = [1053,755,388,166]
                if Habitat == "Rich" : grades = HESrich
                else: grades = HESpoor
                HESgrade = interpret(grades,HES)
      #          print("Grades: ", grades, "HES: ", HES)
                AHESrich=[64.72, 54.56, 45.81, 31.72]
                AHESpoor=[55.69, 45.17, 35.32, 27.49]
                if Habitat == "Rich" : grades = AHESrich
                else: grades = AHESpoor
      #          print("Grades: ", grades, "AHES", AHES)
                AHESgrade=interpret(grades,AHES)
                
    # SemiHES = (HES grade + AHES grade)/2
                SemiHES=(HESgrade+AHESgrade)/2
      #          print("Habitat: ", Habitat, "HESgrade: ", HESgrade, "AHESgrade: ", AHESgrade, "SemiHES: ", SemiHES)

                
    ##
    ##### ΥΠΟΛΟΓΙΣΜΟΣ ΚΑΤΗΓΟΡΙΑΣ ΠΟΙΟΤΗΤΑΣ
    #River Type     REFSemiHES
    #R-M1           4.5
    #R-M2           4.5
    #R-M3           4.5
    #R-M4           5.0
    #R-M5           4.5
                RiverTypes=["R-M1", "R-M2", "R-M3", "R-M4", "R-M5", "R-M6"]
                REFSemiHES=[4.5, 4.5, 4.5, 5.0, 5.0, 4.5]
                bornesHESY2=[["R-M1", 0.94, 0.75,0.5,0.25,0.0],["R-M2",0.94,0.71,0.47,0.24,0.0],["R-M3",0.89,0.67,0.45,0.22,0.0],["R-M4",0.85,0.64,0.43,0.22,0.0],["R-M5",0.96,0.67,0.44,0.22,0.0],["Very Large",0.0,0.0,0.0,0.0,0.0]]
                Qualities=["Υψηλή/High", "Καλή/Good", "Μέτρια/Moderate","Ελλιπής/Poor","Κακή/Bad"]
                Quality_HESY2="Unknown"
                SemiHESforHESY2=SemiHES
                if Rivertype in RiverTypes:
                    index=RiverTypes.index(Rivertype)
#                    if Rivertype == "R-M3" : SemiHESforHESY2=SemiHES/4.5
                    HESY2=SemiHESforHESY2/REFSemiHES[index]
#                    print(bornesHESY2[index])
                    tagsHESY2=bornesHESY2[index]
#                    print(HESY2, tagsHESY2)
#                    print("tags: ",tagsHESY2)
                    for tag in tagsHESY2[1:]:
                        indQuality=tagsHESY2.index(tag)-1
#                        print(tag,indQuality,Qualities[indQuality],type(tag))
                        if type(tag)==float and HESY2 >= tag:
                            Quality_HESY2= Qualities[indQuality]
#                            print("Quality_HESY2: ", Quality_HESY2)
                            break
                else:
                    #print("Αδύνατος ο Υπολογισμός HESY2 για τύπο ρεματος, ", Rivertype)
                    HESY2=0
    #            resultlist=resultlist+[HESY2, Quality]
                
                resultlist=resultlist+[HES,AHES,HESgrade,AHESgrade,SemiHES,HESY2,Quality_HESY2] #   13: HES,14:AHES, 15:HESgrade, 16:AHESgrade, 17:SemiHES, 18:HESY2, 19:Quality
#                print(resultlist)   
                    
                    
    #Ερμηνεία	R-M1	    R-M2	       R-M3             R-M4	    R-M5                Very Large
    #Υψηλή	        ≥0,943	    ≥0,944	    ≥0,889          ≥0,85	    ≥0,963              0.0 
    #Καλή 	    (0,943-0,75]    (0,944-0,708]   (0,889-0,667]   (0,85-0,637]    (0,963-0,673]       0.0
    #Μέτρια	    (0,75-0,5]	    (0,708-0,472]   (0,667-0,445]   (0,637-0,425]   (0,673-0,444]       0.0
    #Ελλιπής    (0,5-0,25]	    (0,472-0,236]   (0,445-0,222]   (0,425-0,213]   (0,444-0,222]       0.0
    #Κακή	    (0,25-0]	    (0,236-0]	    (0,222-0]       (0,213-0]	    (0,222-0]           0.0

                for fam in datafamilies:    
                    ind=datafamilies.index(fam)
                    dataclasses.append(FAMILIES[fam][0])
                    dataorders.append(FAMILIES[fam][1])

    # find the indices of the data of selected family groups (in porders or pclasses) to add
                itoadd=[] # h η θέση της επιλεγμένης οικογένειας στα δεδομένα
                staxa_names=[]
                staxa_values=[]
                staxa_pr100=[]
                staxa_present=[]
                for match in porders:
                    itoadd.append([i for i, x in enumerate(dataorders) if x == match])
                    staxa_names.append(match)
                    GROUPED[match]=[]
                for match in pclasses:
                    itoadd.append([i for i, x in enumerate(dataclasses) if x == match])
                    GROUPED[match]=[]
                    staxa_names.append(match)
    # Add the individuals for each of the selected groups             
                #print("\n\t\t Order or Class \t", "Άτομα \t","Ποσοστό \t", "Συχνότητα")
    #            #print(staxa_names, len(staxa_names))
                for i in range (0,len(staxa_names)):
    #                #print(staxa_names[i],itoadd[i])
                    present=0
                    staxa_values.append(sum(values[x] for x in itoadd[i]))
                    staxa_sum=sum(values[x] for x in itoadd[i])
                    staxa_pr100.append(staxa_sum/sum_atoma*100)
                    GROUPED[staxa_names[i]].append(staxa_sum)
                    GROUPED[staxa_names[i]].append(staxa_sum/sum_atoma*100)
                    for x in itoadd[i]:
                        if values[x]>0 : present=present+1
                    staxa_present.append(present)
                    GROUPED[staxa_names[i]].append(present)
#                    if len(staxa_names[i])<14 :
                         #print("\t\t",staxa_names[i],"\t\t",staxa_values[i],"\t %8.2f"%(staxa_pr100[i]),"\t",staxa_present[i])
#                    else:
#                        #print("\t\t",staxa_names[i],"\t",staxa_values[i],"\t %8.2f"%(staxa_pr100[i]),"\t",staxa_present[i])


    ################## Sensibility categories ----------

    #            SENSIBLES["Sensible"][0]=sum(values[x] if sensitivities[x]=="S")

    ## Ypologismos tou  Log10(Sel_EPTD+1) με                
    # find the indices of the data of the selected Sel_EPTD family groups (in porders or pclasses) to add
                Sel_EPTDs_toadd=[datafamilies.index(match) for match in Sel_EPTDs]
                Sel_EPTDs_sum=sum(values[x] for x in Sel_EPTDs_toadd)
                logEPTDs=math.log10(Sel_EPTDs_sum+1)
                #print("Sel_EPTDs_sum: ",Sel_EPTDs_sum, "Log10(SelEPTDs_sum+1)",logEPTDs)
    #            resultlist=resultlist+[Sel_EPTDs_sum,logEPTDs]
                
               
    #### Υπολογισμός του %EPT (Ephemeroptera+Plecoptera+Trichoptera)
                percent_EPT=GROUPED['Ephemeroptera'][1]+ GROUPED['Plecoptera'][1]+ GROUPED['Trichoptera'][1]
                taxa_EPT=GROUPED['Ephemeroptera'][2]+ GROUPED['Plecoptera'][2]+ GROUPED['Trichoptera'][2]
                #print("taxa_EPT: ", taxa_EPT,"%EPT: ",percent_EPT)
    #            resultlist=resultlist+[taxa_EPT,percent_EPT]
            
    #### Υπολογισμός του %EPTC (Coleoptera+Ephemeroptera+Plecoptera+Trichoptera)
                percent_EPTC=percent_EPT+GROUPED['Coleoptera'][1]
                taxa_EPTC=taxa_EPT+GROUPED['Coleoptera'][2]
                #print("taxa_EPTC: ",taxa_EPTC,"%EPTC: ", percent_EPTC)

                
    #### Υπολογισμός του 1-GOLD (1-((% Diptera+% Oligochaeta+%Gastropoda)/100)
                GOLD = 1-(GROUPED['Diptera'][1]+GROUPED['Oligochaeta'][1]+ GROUPED['Gastropoda'][1])/100
                #print("1-GOLD", GOLD)
    #                                       20      21      22      23      24
                resultlist=resultlist+[ASPTminus2,logEPTDs,taxa_EPT,GOLD,shannon]

    #### Υπολογισμός STAR ICMi (Lazaridou et al. 2018)
    #Metrics STAR ICMi   ASPT-2  Log10(Sel_EPTD+1)	1-GOLD	No of taxa  No of  EPT families	Shannon Diversity index 	STARICMi
    #divide by:         4,552   2,097	        0,914	    27	    11	                1,886	                        0,999
    #Weights:           0,333   0,266               0,067       0,167   0,083               0,083
    #                   "Υψηλή/High",   "Καλή/Good", "Μέτρια/Moderate", "Ελλιπής/Poor" "Κακή/Bad"
    #Bornes:                ≥1.01       (1.01-0.73]    (0.73-0.53]      (0.53-0.35]     (0.35-0]
#                STAR_ICMi = 'NaN' #+ Rivertype
#                Quality_STAR_ICMi = 'N/A for '+ Rivertype
                if Rivertype == "Very large":
                    Rivertype = "Very Large"
#                print(Rivertype)
                STARdivideby={"R-M1":(4.86944444444445,1.98447484049067,0.800927870229408,25,11,5,2.17420470409286,0.993333378630599),
                              "R-M2":(4.89117647058823,1.96613538794975,0.82112404881111,24,11,2.10613303787625,0.969698144195807),
                              "R-M3":(4.55172413793104,2.09691001300806,0.913541255428346,27,11,1.88642448112745,0.999327106715808),
                              "R-M4":(4.9047619047619,2.05307844348342,0.8212232096184,23,12,2.1285085703782,1.01500647537176),
                              "R-M5":(5.19264705882353,1.6066260260982,0.711559714138363,20,11,5,1.75422584184246,1.00920763927443),
                              "Very Large":(4.552,2.097,0.914,27,11,1.886,0.99)}
#                print(STARdivideby[Rivertype])
                STARmetrics=[ASPTminus2,logEPTDs,GOLD,taxa,taxa_EPT,shannon]
                a=STARmetrics
                b=STARdivideby[Rivertype][0:6]
                STARmetrics=[a/b for a, b in zip(a, b)]
                weights=[0.333,0.266,0.067,0.167,0.083,0.083]
                a=STARmetrics
  #              print(a)
                b=weights
                STARmetrics=[a*b for a, b in zip(a, b)]
  #              print(STARmetrics)
                STAR_ICMi=sum(STARmetrics)
  #              print("STAR_ICMi: ", STAR_ICMi, "STARdivideby[6]: ",STARdivideby[6])
                EQR_STAR_ICMi=STAR_ICMi/STARdivideby[Rivertype][6]
                Qualities_STAR_ICMi=["Πολύ καλή/Very good", "Καλή/Good", "Μέτρια/Moderate","Ελλιπής/Poor","Κακή/Bad"]
                tags_STAR_ICMi=[1.01, 0.73, 0.53, 0.35,0.0]
            
                for tag in tags_STAR_ICMi:
                    indQuality_STAR_ICMi=tags_STAR_ICMi.index(tag)
#                #print(tag,indQuality_IBMWP,type(tag))
                    if type(tag)==float and STAR_ICMi >= tag:
                        Quality_STAR_ICMi= Qualities_STAR_ICMi[indQuality_STAR_ICMi]
                        #print("Quality_STAR_ICMi: ", Quality_STAR_ICMi)
                        break            
                #print("STAR_ICMi: ",STAR_ICMi,"Quality_STAR_ICMi: ", Quality_STAR_ICMi)
                if Rivertype !="Very Large":
                    Quality_STAR_ICMi = ""                   
           
    #                                       25          26
                resultlist=resultlist+[STAR_ICMi,Quality_STAR_ICMi]

    #### Υπολογισμός Quality IBMWP
    #ΙBMWP score	Category
    #< 15	Very poor
    #15-35	Poor
    #36-60	Moderate
    #61-100	Good
    #>100	Very good

                Qualities_IBMWP=["Πολύ καλή/Very good", "Καλή/Good", "Μέτρια/Moderate","Ελλιπής/Poor","Κακή/Bad"]
                tags_IBMWP=[100.0, 60.0, 35.0, 15.0,0.0]
                for tag in tags_IBMWP:
                    indQuality_IBMWP=tags_IBMWP.index(tag)
    #                #print(tag,indQuality_IBMWP,type(tag))
                    if type(tag)==float and IBMWP >= tag:
                        Quality_IBMWP= Qualities_IBMWP[indQuality_IBMWP]
                        #print("Quality_IBMWP: ", Quality_IBMWP)
                        break

    #                                   27     28      29        
                resultlist=resultlist+[IBMWP,IASPT,Quality_IBMWP]

    #### Υπολογισμός Quality BMWP
    #Για τον δείκτη BMWP η ερμηνεία είναι 
    #BMWP score	Category	Interpretation
    #0-10	        Very poor	Heavily polluted
    #11-40	        Poor	        Polluted or impacted
    #41-70	        Moderate	Moderately impacted
    #71-100	        Good	        Clean but slightly impacted
    #>100	        Very good	Unpolluted, unimpacted

                Qualities_BMWP=["Πολύ καλή/Very good", "Καλή/Good", "Μέτρια/Moderate","Ελλιπής/Poor","Κακή/Bad"]
                tags_BMWP=[100.0, 70.0, 40.0, 10.0,0.0]
                for tag in tags_BMWP:
                    indQuality_BMWP=tags_BMWP.index(tag)
    #               #print(tag,indQuality,type(tag))
                    if type(tag)==float and BMWP >= tag:
                        Quality_BMWP= Qualities_BMWP[indQuality_BMWP]
                        #print("Quality_BMWP: ", Quality_BMWP)
                        break

    #                           30      31          32  
                resultlist=resultlist+[BMWP,ASPT,Quality_BMWP]

    #### GROUPED 

    #                                           33                  34                              35                      36                          37          
                resultlist=resultlist+[GROUPED['Ephemeroptera'][1],GROUPED['Trichoptera'][1],GROUPED['Coleoptera'][1],GROUPED['Plecoptera'][1],GROUPED['Diptera'][1]]
    #                                       38                          39                      40        
                resultlist=resultlist+[GROUPED['Odonata'][1],GROUPED['Gastropoda'][1],GROUPED['Oligochaeta'][1]]

    #                                     41                          42                        43                              44                       45                     
                resultlist=resultlist+[GROUPED['Ephemeroptera'][2],GROUPED['Trichoptera'][2],GROUPED['Coleoptera'][2],GROUPED['Plecoptera'][2], GROUPED['Diptera'][2]]

    #                                    46             47
                resultlist=resultlist+[GROUPED['Odonata'][2],taxa_EPTC]
    #                                  48          49
                resultlist=resultlist+[percent_EPT, percent_EPTC]

                result.append(resultlist)
    RES_window()
#    bSwitch("Input",'hidden')
    bSwitch("Datashow",'normal')
    bSwitch("Metrics",'normal')
    bSwitch("Export",'normal')
    bSwitch("Clear",'normal')

def frozenshow(filename, toshow):
### File name= string όνομα αρχειου
### toshow = pandas dataframe    
##  Μετατρέπει το dataframe σε QTableView
##  και στη σνέχεια τροφοδοτή την FreezeTableWidget(model)
##  που φτιάχνει tableview με frozen στήλες και γραμμές
#    print(toshow[0:3])
    model = pandasModel(toshow) ## Μετατροπή του pd.dataframe σε QTableView για εισαγωγή στην επόμενη κλάση                    
    app = QApplication(sys.argv)
    tableView = FreezeTableWidget(model) # κλάση που φτιάχνει Tableview με frozen 1η στήλη και γραμμή.   
#    print("tableview is OK")
    tableView.setWindowTitle(filename)
    tableView.resize(900, 680)
    tableView.show()
    app.exec_()


def RES_window():
    filename = os.path.basename(rFileName)
    output=[list(x) for x in zip(*result)]
    df=pd.DataFrame(output)
    for i in range(1,len(df.columns)):
       for j in range(0,len(df[i])):
            a=df[i][j]
#            print(i, j, a, type(a))  
            if j==2 and type(a)==float: #convert excel date to dd/mm/yyyy format
                a=xlrd.xldate_as_datetime(a, 0).date().strftime("%d/%m/%Y")
                df[i][j]=a
            if type(a)!= int and type(a)!=str:
                a = float("{:.2f}".format(a))
                a=format(a, ".2f")
                df[i][j]=a
            if j==4 :
                df[i][j]=str(df[i][j-3])+"\n"+str(df[i][j-2])+"\n"+str(df[i][j-1])+"\n"+str(df[i][j])
    df[0][4]="Sample\nDate\nRiver type\nHabitat"
#    df[0:3]
    df, df.columns = df[5:] , df.iloc[4]
#### Χρησιμοποιήθηκαν για να γίνει transposed
## αλλά υπήρξαν προβλήματα με το ύψος γραμμών στο frozenview
#    df=df.T
#    df, df.columns = df[1:] , df.iloc[0]
#    df[0:3]

#    df[0:3]
#    print("data: ", type(data))
    frozenshow("Metrics for "+filename, df)

def exporttoexcel():
    global dfout
    filename = os.path.basename(rFileName)
    if "Samples" or "SAMPLES" in filename[0:6]:
        filename=filename[7:len(filename)]
#	print(filename)
    filename=filename[0:len(filename)-4]+".xls" ### Γιατί δεν κατάλαβε αρχειο που τελειώνει σε ".XLS" κεφαλαία
#    print(filename)
# Routine to save the results in an excel file
    output=[list(x) for x in zip(*result)]
    dfout = pd.DataFrame(output)
#    print(dfout)
# Specify a writer
    wFilename='RESULTS_'+filename
    writer = pd.ExcelWriter(wFilename, engine='xlsxwriter')

# Write your DataFrame to a file     

    dfout.to_excel(writer, 'HESY2-output', header=False, index=False)
    workbook=writer.book
    format=workbook.add_format({'text_wrap': True, 'num_format' : "##0.00"})
    format.set_align('vcenter')
    
    format_center_bold=workbook.add_format({'bold':True, 'text_wrap': True})
    format_center_bold.set_align('center')
    format_center_bold.set_align('vcenter')
    
    format_center=workbook.add_format({'text_wrap': True})
    format_center.set_align('vcenter')
#    format_center.set_align('center')

    date_format = workbook.add_format({'num_format': 'dd/mm/yyyy', 'align': 'center', 'bold':True})

    format1=format_center
    format1.set_num_format("#,###.##")
    
    format2=format_center
    format2.set_num_format("#,###")

    worksheet=writer.sheets['HESY2-output']    
		
  
    worksheet=writer.sheets['HESY2-output']
    worksheet.set_column('A:XFD',20,format)
    for i in range(0,5):
        worksheet.set_row(i,None,format_center_bold)
    for i in (19,26,29,32):
        worksheet.set_row(i,None,format_center_bold)
    for i in (5,6,7,8,9,13,15,16,22,27,30,41,42,43,44,45,46,47):
        worksheet.set_row(i,None,format2)
    worksheet.set_row(2,None,date_format)
# Save the result 
    writer.save()

def clear():
    wb=[]
    filename=""
    dfout=[]
    dfin=[]
    df=[]
    data=[]
    bSwitch("Input",'normal')
    bSwitch("Datashow",'hidden')
    bSwitch("Metrics",'hidden')
    bSwitch("Export", 'hidden')
    bSwitch("Clear",'hidden')
    file_label.destroy()
    
def stop():
    root.destroy()

def change_color():
    current_color = empty_image.cget("background")
    next_color = "#33B5E5" if current_color == "red" else "red"
    empty_image.config(background=next_color)
    root.after(1000, change_color)

labels=[]
buttons=[]
wb=[]
counter = 0
current_window = None
#filename=""

root = tk.Tk()
root.title("HESY2 & VLR, Aristotle University of Thessaloniki, Faculty of Science, School of Biology, Dept of Zoology © Maria Lazaridou 2020 version 1.1")
#### UPDATE 15/12/2020
FILENAME='Cover_2020_V_1_1.jpg'
noimage=0
try:
    img = Image.open(FILENAME)
    canvas = tk.Canvas(root, width=img.width, height=img.height)
    canvas.pack()
    tk_img = ImageTk.PhotoImage(file = FILENAME)
    canvas.create_image(img.width/2, img.height/2, image=tk_img) # id=1
except:
    canvas = tk.Canvas(root, width=900, height=700)
    canvas.pack()
    empty_image = tk.Label(root, text ="Background image "+FILENAME+" not found", anchor = 'c',
                         width = 50, activebackground = "red", font=('Arial', '14', 'bold'))
    empty_image_window=canvas.create_window(446, 250, anchor='c', window=empty_image)
    noimage=1
#    empty_image.pack()
#######background = "white",
root.wm_attributes("-transparentcolor", 'grey')
# id = 2
link1 = tk.Label(root, text="http://fishlab.bio.auth.gr/", fg="blue", cursor="hand2", background = "white", font=('Arial', '13', "underline"))
link1.pack()
link1.bind("<Button-1>", callback1)
link1_window=canvas.create_window(25, 620, anchor='w', window=link1)    

# id = 3
link2 = tk.Label(root, text="http://river.bio.auth.gr/", fg="blue", cursor="hand2", background = "white", font=('Arial', '13', "underline"))
link2.pack()
link2.bind("<Button-1>", callback2)
link2_window=canvas.create_window(25, 642, anchor='w', window=link2)

# id = 4    
next_button = tk.Button(root, text = "Import a sample\ndata file", command = input_choose, anchor = 'c',
                     width = 15, activebackground = "#33B5E5", font=('Arial', '18', 'bold'))
next_button_window = canvas.create_window(446, 415, anchor='c', window=next_button)   
#canvas.itemconfigure(4, state='normal') # Input
canvas.itemconfigure(4, state='normal') # Input
#print(next_button_window)

# id = 5
sample_button = tk.Button(root, text = "Show\nsample data", command = sampleShow, anchor = 'c',
                     width = 10, activebackground = "#33B5E5", font=('Arial', '18', 'bold'))
sample_button_window = canvas.create_window(360, 425, anchor='c', window=sample_button)    
#canvas.itemconfigure(5, state='hidden') # Datashow
canvas.itemconfigure(5, state='normal') # Datashow
#print(sample_button_window)

# id = 6
calculate_button = tk.Button(current_window, text = "Show Metrics", command = Metrics, anchor = 'c',
                     width = 15, activebackground = "#33B5E5", font=('Arial', '18', 'bold'))
calculate_button_window = canvas.create_window(580, 390, anchor='c', window=calculate_button)    
#canvas.itemconfigure(6, state='hidden') # Metrics button
canvas.itemconfigure(6, state='normal') # Metrics button
#print(calculate_button_window)

# id = 7 
save_button = tk.Button(current_window, text = "Export Metrics", command = exporttoexcel, anchor = 'c',
                     width = 15, activebackground = "#33B5E5", font=('Arial', '18', 'bold'))
save_button_window = canvas.create_window(580, 450, anchor='c', window=save_button)    
#canvas.itemconfigure(7, state='hidden') # Export
canvas.itemconfigure(7, state='normal') # Export
#print(save_button_window)

# id = 8
clear_button = tk.Button(root, text = "Reset", command = clear, anchor = 'c',
                     width = 5, activebackground = "#33B5E5", font=('Arial', '18', 'bold'))
clear_button_window = canvas.create_window(446, 520, anchor='c', window=clear_button)
#canvas.itemconfigure(8, state='hidden') # Clear
canvas.itemconfigure(8, state='normal') # Clear
#print(clear_button_window)

### id = 9
quit_button = tk.Button(root, text = "Quit", command = stop, anchor = 'c',
                     width = 5, activebackground = "#33B5E5", font=('Arial', '18', 'bold'))
quit_button_window = canvas.create_window(446, 580, anchor='c', window=quit_button)
#canvas.itemconfigure(9, state='hidden') # Quit
canvas.itemconfigure(9, state='normal') # Quit
#print(quit_button_window)

# id = 10
try:
    file_label = tk.Label(root, text = "Imported data file: "+filename, anchor = 'w',
                         width = 50, activebackground = "#33B5E5", font=('Arial', '14', 'bold'))
    file_label_window = canvas.create_window(470, 350, anchor='c', window=file_label)
    canvas.itemconfigure(10, state='normal') # File
#    print(file_label_window)
except:
    pass

bSwitch("Input",'normal')
if noimage==1 :
    bSwitch("Input",'hidden')
    change_color()
bSwitch("Datashow",'hidden')
bSwitch("Metrics",'hidden')
bSwitch("Export", 'hidden')
bSwitch("Clear",'hidden')
bSwitch("Quit",'normal')
bSwitch("File",'hidden')

root.mainloop()

        
        
        


